/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_services.c: Services aliases 
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_services.c,v 1.1 2002/08/26 21:34:10 strtok Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "vchannel.h"
#include "client.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_user.h"
#include "s_conf.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "packet.h"
#include "common.h"

static void m_nickserv(struct Client*, struct Client*, int, char**);
static void m_chanserv(struct Client*, struct Client*, int, char**);
static void m_memoserv(struct Client*, struct Client*, int, char**);
static void m_operserv(struct Client*, struct Client*, int, char**);

static void do_servalias(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[], char *type);


struct Message nickserv_msgtab = {
  "NICKSERV", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_nickserv, m_nickserv, m_nickserv}
};
struct Message ns_msgtab = {
  "NS", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_nickserv, m_nickserv, m_nickserv}
};

struct Message operserv_msgtab = {
  "OPERSERV", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_operserv, m_operserv, m_operserv}
};
struct Message os_msgtab = {
  "OS", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_operserv, m_operserv, m_operserv}
};

struct Message chanserv_msgtab = {
  "CHANSERV", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_chanserv, m_chanserv, m_chanserv}
};
struct Message cs_msgtab = {
  "CS", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_chanserv, m_chanserv, m_chanserv}
};

struct Message memoserv_msgtab = {
  "MEMOSERV", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_memoserv, m_memoserv, m_memoserv}
};
struct Message ms_msgtab = {
  "MS", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_memoserv, m_memoserv, m_memoserv}
};


#ifndef STATIC_MODULES

void
_modinit(void)
{
  mod_add_cmd(&nickserv_msgtab);
  mod_add_cmd(&ns_msgtab);
  mod_add_cmd(&operserv_msgtab);
  mod_add_cmd(&os_msgtab);
  mod_add_cmd(&chanserv_msgtab);
  mod_add_cmd(&cs_msgtab);
  mod_add_cmd(&memoserv_msgtab);
  mod_add_cmd(&cs_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&nickserv_msgtab);
  mod_del_cmd(&ns_msgtab);
  mod_del_cmd(&operserv_msgtab);
  mod_del_cmd(&os_msgtab);
  mod_del_cmd(&chanserv_msgtab);
  mod_del_cmd(&cs_msgtab);
  mod_del_cmd(&memoserv_msgtab);
  mod_del_cmd(&cs_msgtab);
}


const char *_version = "$Revision: 1.1 $";
#endif

static void m_nickserv(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{
   do_servalias(client_p, source_p, parc, parv, "NICKSERV");
}

static void m_operserv(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{
   do_servalias(client_p, source_p, parc, parv, "OPERSERV");
}

static void m_chanserv(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{
   do_servalias(client_p, source_p, parc, parv, "CHANSERV");
}

static void m_memoserv(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{
   do_servalias(client_p, source_p, parc, parv, "MEMOSERV");
}


static void do_servalias(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[], char *type)
{
   struct Client *target_p;
   int i;
 
   if(!IsPerson(source_p))
      return;

   /* Recreate string, this is because dalnet let the services
      aliases through without a :, which meant clients were
      never standardized */

   for(i = 2; i < parc; i++)
      *(parv[i] - 1) = ' ';

   if ((target_p = find_client(type)) == NULL)
   {
      sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name,
                 parv[0], type);
      return;
   }
   
   if(!IsService(target_p))
   {
      sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name,
                 parv[0], type);
      return;
   }

   sendto_one(target_p, ":%s PRIVMSG %s :%s", source_p->name, target_p->name,
              parv[1]);

}
